import {SimpleState} from "../../game-model/gamesystems/states/SimpleState";
import {ScriptAccountConditionParser} from "./ScriptAccountConditionParser";
import {ScriptAccount} from "../../game-model/scriptAccounts/ScriptAccount";
import {SimpleTransition} from "../../game-model/gamesystems/transitions/SimpleTransition";
import {ScriptAccountActionParser} from "./ScriptAccountActionParser";


export class TransitionParser {

  private states: SimpleState[]
  private conditionParser: ScriptAccountConditionParser
  private actionParser: ScriptAccountActionParser


  constructor(states: SimpleState[], scriptAccounts: ScriptAccount[]) {
    this.states = states;
    this.conditionParser = new ScriptAccountConditionParser(scriptAccounts);
    this.actionParser = new ScriptAccountActionParser(scriptAccounts)
  }

  public parseTransitions(transitionData: any): SimpleTransition[] {
    const transitions: SimpleTransition[] = []
    for(let i=0; i<transitionData.length; i++) {
      const parsedTransition = this.parseSingleTransition(transitionData[i]);
      if(parsedTransition != undefined) {
        transitions.push(parsedTransition)
      }
    }
    return transitions
  }

  private parseSingleTransition(transitionData: any): SimpleTransition | undefined{
    const startingState = this.findStateByLabel(transitionData.startingState)
    const endingState = this.findStateByLabel(transitionData.endingState);

    if(startingState == undefined || endingState == undefined) {
      return undefined
    }

    const simpleTransition = new SimpleTransition(startingState, endingState)
    simpleTransition.scriptAccountConditions = this.conditionParser.parseStoredConditions(transitionData.scriptAccountConditions)
    simpleTransition.scriptAccountActions = this.actionParser.parseActions(transitionData.scriptAccountActions);

    return simpleTransition;
  }

  private findStateByLabel(stateLabel: string) {
    return this.states.find(state => state.stateLabel === stateLabel);
  }
}
